import os
import logging
import pytest
from jenkinsapi.jenkins import Jenkins
from jenkinsapi_utils.jenkins_launcher import JenkinsLancher

log = logging.getLogger(__name__)
state = {}

# User/password for authentication testcases
ADMIN_USER = 'admin'
ADMIN_PASSWORD = 'admin'

# Extra plugins required by the systests
PLUGIN_DEPENDENCIES = [
    "http://updates.jenkins.io/latest/"
    "apache-httpcomponents-client-4-api.hpi",
    "http://updates.jenkins.io/latest/jsch.hpi",
    "http://updates.jenkins.io/download/plugins/trilead-api/1.0.5/"
    "trilead-api.hpi",
    "http://updates.jenkins.io/latest/workflow-api.hpi",
    "http://updates.jenkins.io/latest/display-url-api.hpi",
    "http://updates.jenkins.io/latest/workflow-step-api.hpi",
    "http://updates.jenkins.io/latest/workflow-scm-step.hpi",
    "http://updates.jenkins.io/latest/icon-shim.hpi",
    "http://updates.jenkins.io/download/plugins/junit/1.28/junit.hpi",
    "http://updates.jenkins.io/latest/script-security.hpi",
    "http://updates.jenkins.io/latest/matrix-project.hpi",
    "http://updates.jenkins.io/latest/credentials.hpi",
    "http://updates.jenkins.io/latest/ssh-credentials.hpi",
    "http://updates.jenkins.io/latest/scm-api.hpi",
    "http://updates.jenkins.io/latest/mailer.hpi",
    "http://updates.jenkins.io/latest/git.hpi",
    "http://updates.jenkins.io/latest/git-client.hpi",
    "https://updates.jenkins.io/latest/nested-view.hpi",
    "https://updates.jenkins.io/latest/ssh-slaves.hpi",
    "https://updates.jenkins.io/latest/structs.hpi",
    "http://updates.jenkins.io/latest/plain-credentials.hpi",
    "http://updates.jenkins.io/latest/envinject.hpi",
    "http://updates.jenkins.io/latest/envinject-api.hpi",
    "http://updates.jenkins.io/latest/jdk-tool.hpi"
]


def _delete_all_jobs(jenkins):
    jenkins.poll()
    for name in jenkins.keys():
        del jenkins[name]


def _delete_all_views(jenkins):
    all_view_names = jenkins.views.keys()[1:]
    for name in all_view_names:
        del jenkins.views[name]


def _delete_all_credentials(jenkins):
    all_cred_names = jenkins.credentials.keys()
    for name in all_cred_names:
        del jenkins.credentials[name]


def _create_admin_user(launched_jenkins):
    # Groovy script that creates a user "admin/admin" in jenkins
    # and enable security. "admin" user will be the only user and
    # have admin permissions. Anonymous cannot read anything.
    create_admin_groovy = """
import jenkins.model.*
import hudson.security.*

def instance = Jenkins.getInstance()

def hudsonRealm = new HudsonPrivateSecurityRealm(false)
hudsonRealm.createAccount('{0}','{1}')
instance.setSecurityRealm(hudsonRealm)

def strategy = new FullControlOnceLoggedInAuthorizationStrategy()
strategy.setAllowAnonymousRead(false)
instance.setAuthorizationStrategy(strategy)
    """.format(ADMIN_USER, ADMIN_PASSWORD)

    url = launched_jenkins.jenkins_url
    jenkins_instance = Jenkins(url)
    jenkins_instance.run_groovy_script(create_admin_groovy)


def _disable_security(launched_jenkins):
    # Groovy script that disables security in jenkins,
    # reverting the changes made in "_create_admin_user" function.
    disable_security_groovy = """
import jenkins.model.*
import hudson.security.*

def instance = Jenkins.getInstance()
instance.disableSecurity()
instance.save()
    """

    url = launched_jenkins.jenkins_url
    jenkins_instance = Jenkins(url, ADMIN_USER, ADMIN_PASSWORD)
    jenkins_instance.run_groovy_script(disable_security_groovy)


@pytest.fixture(scope='session')
def launched_jenkins():
    systests_dir, _ = os.path.split(__file__)
    local_orig_dir = os.path.join(systests_dir, 'localinstance_files')
    if not os.path.exists(local_orig_dir):
        os.mkdir(local_orig_dir)
    war_name = 'jenkins.war'
    launcher = JenkinsLancher(
        local_orig_dir, systests_dir, war_name, PLUGIN_DEPENDENCIES,
        jenkins_url=os.getenv('JENKINS_URL', None)
    )
    launcher.start()

    yield launcher

    log.info('All tests finished')
    launcher.stop()


@pytest.fixture(scope='function')
def jenkins(launched_jenkins):
    url = launched_jenkins.jenkins_url

    jenkins_instance = Jenkins(url, timeout=30)

    _delete_all_jobs(jenkins_instance)
    _delete_all_views(jenkins_instance)
    _delete_all_credentials(jenkins_instance)

    return jenkins_instance


@pytest.fixture(scope='function')
def lazy_jenkins(launched_jenkins):
    url = launched_jenkins.jenkins_url

    jenkins_instance = Jenkins(url, lazy=True)

    _delete_all_jobs(jenkins_instance)
    _delete_all_views(jenkins_instance)
    _delete_all_credentials(jenkins_instance)

    return jenkins_instance


@pytest.fixture(scope='function')
def jenkins_admin_admin(launched_jenkins, jenkins):  # pylint: disable=unused-argument
    # Using "jenkins" fixture makes sure that jobs/views/credentials are
    # cleaned before security is enabled.
    url = launched_jenkins.jenkins_url

    _create_admin_user(launched_jenkins)
    jenkins_admin_instance = Jenkins(url, ADMIN_USER, ADMIN_PASSWORD)

    yield jenkins_admin_instance

    jenkins_admin_instance.requester.__class__.AUTH_COOKIE = None
    _disable_security(launched_jenkins)



"""
This example shows how to create job from XML file and how to delete job
"""
from __future__ import print_function
from pkg_resources import resource_string
from jenkinsapi.jenkins import Jenkins

jenkins = Jenkins('http://localhost:8080')
job_name = 'foo_job2'
xml = resource_string('examples', 'addjob.xml')

print(xml)

job = jenkins.create_job(jobname=job_name, xml=xml)

# Get job from Jenkins by job name
my_job = jenkins[job_name]
print(my_job)

# Delete job using method in Jenkins class
#
# Another way is to use:
#
# del jenkins[job_name]
jenkins.delete_job(job_name)



"""
jenkinsapi plugins
"""
from __future__ import print_function

import logging
import time
import re
try:
    from StringIO import StringIO
    from urllib import urlencode
except ImportError:
    # Python3
    from io import BytesIO as StringIO
    from urllib.parse import urlencode
import json
import requests
from jenkinsapi.plugin import Plugin
from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.custom_exceptions import UnknownPlugin
from jenkinsapi.custom_exceptions import JenkinsAPIException
from jenkinsapi.utils.jsonp_to_json import jsonp_to_json
from jenkinsapi.utils.manifest import read_manifest


log = logging.getLogger(__name__)


class Plugins(JenkinsBase):

    """
    Plugins class for jenkinsapi
    """

    def __init__(self, url, jenkins_obj):
        self.jenkins_obj = jenkins_obj
        JenkinsBase.__init__(self, url)
        # print('DEBUG: Plugins._data=', self._data)

    def get_jenkins_obj(self):
        return self.jenkins_obj

    def check_updates_server(self):
        url = (
            '%s/pluginManager/checkUpdatesServer'
            % self.jenkins_obj.baseurl
        )
        self.jenkins_obj.requester.post_and_confirm_status(
            url, params={}, data={})

    @property
    def update_center_dict(self):
        update_center = 'https://updates.jenkins.io/update-center.json'
        jsonp = requests.get(update_center).content.decode('utf-8')
        return json.loads(jsonp_to_json(jsonp))

    def _poll(self, tree=None):
        return self.get_data(self.baseurl, tree=tree)

    def keys(self):
        return self.get_plugins_dict().keys()

    __iter__ = keys

    def iteritems(self):
        return self._get_plugins()

    def values(self):
        return [a[1] for a in self.iteritems()]

    def _get_plugins(self):
        if 'plugins' in self._data:
            for p_dict in self._data["plugins"]:
                yield p_dict["shortName"], Plugin(p_dict)

    def get_plugins_dict(self):
        return dict(self._get_plugins())

    def __len__(self):
        return len(self.get_plugins_dict().keys())

    def __getitem__(self, plugin_name):
        try:
            return self.get_plugins_dict()[plugin_name]
        except KeyError:
            raise UnknownPlugin(plugin_name)

    def __setitem__(self, shortName, plugin):
        """
        Installs plugin in Jenkins.

        If plugin already exists - this method is going to uninstall the
        existing plugin and install the specified version if it is not
        already installed.

        :param shortName: Plugin ID
        :param plugin a Plugin object to be installed.
        """
        if self.plugin_version_already_installed(plugin):
            return
        if plugin.is_latest(self.update_center_dict):
            self._install_plugin_from_updatecenter(plugin)
        else:
            self._install_specific_version(plugin)
        self._wait_until_plugin_installed(plugin)

    def _install_plugin_from_updatecenter(self, plugin):
        """
        Latest versions of plugins can be installed from the update
        center (and don't need a restart.)
        """
        xml_str = plugin.get_attributes()
        url = (
            '%s/pluginManager/installNecessaryPlugins'
            % self.jenkins_obj.baseurl
        )
        self.jenkins_obj.requester.post_xml_and_confirm_status(
            url, data=xml_str)

    @property
    def update_center_install_status(self):
        """
        Jenkins 2.x specific
        """
        url = "%s/updateCenter/installStatus" % self.jenkins_obj.baseurl
        status = self.jenkins_obj.requester.get_url(url)
        if status.status_code == 404:
            raise JenkinsAPIException(
                'update_center_install_status not available for Jenkins 1.X')
        return status.json()

    @property
    def restart_required(self):
        """
        Call after plugin installation to check if Jenkins requires a restart
        """
        try:
            jobs = self.update_center_install_status['data']['jobs']
        except JenkinsAPIException:
            return True  # Jenkins 1.X has no update_center
        return any([job for job in jobs if job['requiresRestart'] == 'true'])

    def _install_specific_version(self, plugin):
        """
        Plugins that are not the latest version have to be uploaded.
        """
        download_link = plugin.get_download_link(
            update_center_dict=self.update_center_dict)
        downloaded_plugin = self._download_plugin(download_link)
        plugin_dependencies = self._get_plugin_dependencies(downloaded_plugin)
        log.debug("Installing dependencies for plugin '%s'", plugin.shortName)
        self.jenkins_obj.install_plugins(plugin_dependencies)
        url = ('%s/pluginManager/uploadPlugin' % self.jenkins_obj.baseurl)
        requester = self.jenkins_obj.requester
        downloaded_plugin.seek(0)
        requester.post_and_confirm_status(
            url, files={'file': ('plugin.hpi', downloaded_plugin)},
            data={}, params={})

    def _get_plugin_dependencies(self, downloaded_plugin):
        """
        Returns a list of all dependencies for a downloaded plugin
        """
        plugin_dependencies = []
        manifest = read_manifest(downloaded_plugin)
        manifest_dependencies = manifest.main_section.get(
            'Plugin-Dependencies')
        if manifest_dependencies:
            dependencies = manifest_dependencies.split(',')
            for dep in dependencies:
                # split plugin:version;resolution:optional entries
                components = dep.split(';')
                dep_plugin = components[0]
                name = dep_plugin.split(':')[0]
                # install latest dependency, avoids multiple
                # versions of the same dep
                plugin_dependencies.append(
                    Plugin({'shortName': name, 'version': 'latest'}))
        return plugin_dependencies

    def _download_plugin(self, download_link):
        downloaded_plugin = StringIO()
        downloaded_plugin.write(requests.get(download_link).content)
        return downloaded_plugin

    def _plugin_has_finished_installation(self, plugin):
        """
        Return True if installation is marked as 'Success' or
        'SuccessButRequiresRestart' in Jenkins' update_center,
        else return False.
        """
        try:
            jobs = self.update_center_install_status['data']['jobs']
            for job in jobs:
                if job['name'] == plugin.shortName and \
                        job['installStatus'] \
                        in ['Success', 'SuccessButRequiresRestart']:
                    return True
            return False
        except JenkinsAPIException:
            return False  # lack of update_center in Jenkins 1.X

    def plugin_version_is_being_installed(self, plugin):
        """
        Return true if plugin is currently being installed.
        """
        try:
            jobs = self.update_center_install_status['data']['jobs']
        except JenkinsAPIException:
            return False  # lack of update_center in Jenkins 1.X
        return any([job for job in jobs
                    if job['name'] == plugin.shortName
                    and job['version'] == plugin.version])

    def plugin_version_already_installed(self, plugin):
        """
        Check if plugin version is already installed
        """
        if plugin.shortName not in self:
            if self.plugin_version_is_being_installed(plugin):
                return True
            return False
        installed_plugin = self[plugin.shortName]
        if plugin.version == installed_plugin.version:
            return True
        elif plugin.version == "latest":
            # we don't have an exact version, we first check if Jenkins
            # knows about an update
            if hasattr(installed_plugin, 'hasUpdates') \
                    and installed_plugin.hasUpdates:
                return False

            # Jenkins may not have an up-to-date catalogue,
            # so check update-center directly
            latest_version = self.update_center_dict[
                'plugins'][plugin.shortName]['version']
            return installed_plugin.version == latest_version

        return False

    def __delitem__(self, shortName):
        if re.match('.*@.*', shortName):
            real_shortName = re.compile('(.*)@(.*)').search(shortName).group(1)
            raise ValueError(
                ("Plugin shortName can't contain version. '%s' should be '%s'")
                % (shortName, real_shortName)
            )
        if shortName not in self:
            raise KeyError(
                'Plugin with ID "%s" not found, cannot uninstall' % shortName)
        if self[shortName].deleted:
            raise JenkinsAPIException(
                'Plugin "%s" already marked for uninstall. '
                'Restart jenkins for uninstall to complete.')
        params = {
            'Submit': 'OK',
            'json': {}
        }
        url = ('%s/pluginManager/plugin/%s/doUninstall'
               % (self.jenkins_obj.baseurl, shortName))
        self.jenkins_obj.requester.post_and_confirm_status(
            url, params={}, data=urlencode(params)
        )

        self.poll()
        if not self[shortName].deleted:
            raise JenkinsAPIException(
                "Problem uninstalling plugin '%s'." % shortName)

    def _wait_until_plugin_installed(self, plugin, maxwait=120, interval=1):
        for _ in range(maxwait, 0, -interval):
            self.poll()
            if self._plugin_has_finished_installation(plugin):
                return True
            if plugin.shortName in self:
                return True  # for Jenkins 1.X
            time.sleep(interval)

        if self.jenkins_obj.version.startswith('2'):
            raise JenkinsAPIException(
                "Problem installing plugin '%s'." % plugin.shortName)

        log.warning("Plugin '%s' not found in loaded plugins."
                    "You may need to restart Jenkins.", plugin.shortName)
        return False

    def __contains__(self, plugin_name):
        """
        True if plugin_name is the name of a defined plugin
        """
        return plugin_name in self.keys()

    def __str__(self):
        plugins = [plugin["shortName"]
                   for plugin in self._data.get("plugins", [])]
        return str(sorted(plugins))



"""
System tests for `jenkinsapi.jenkins` module.
"""
from jenkinsapi_tests.systests.job_configs import JOB_WITH_ENV_VARS
from jenkinsapi_tests.test_utils.random_strings import random_string


def test_get_env_vars(jenkins):
    job_name = 'get_env_vars_create1_%s' % random_string()
    job = jenkins.create_job(job_name, JOB_WITH_ENV_VARS)
    job.invoke(block=True)
    build = job.get_last_build()
    while build.is_running():
        time.sleep(0.25)
    data = build.get_env_vars()
    assert data['key1'] == 'value1'
    assert data['key2'] == 'value2'



"""
Module for jenkinsapi Credential class
"""
import logging
import xml.etree.cElementTree as ET

log = logging.getLogger(__name__)


class Credential(object):
    """
    Base abstract class for credentials

    Credentials returned from Jenkins don't hold any sensitive information,
    so there is nothing useful can be done with existing credentials
    besides attaching them to Nodes or other objects.

    You can create concrete Credential instance: UsernamePasswordCredential or
    SSHKeyCredential by passing credential's description and credential dict.

    Each class expects specific credential dict, see below.
    """
    # pylint: disable=unused-argument

    def __init__(self, cred_dict, jenkins_class=''):
        """
        Create credential

        :param str description: as Jenkins doesn't allow human friendly names
            for credentials and makes "displayName" itself,
            there is no way to find credential later,
            this field is used to distinguish between credentials
        :param dict cred_dict: dict containing credential information
        """
        self.credential_id = cred_dict.get('credential_id', '')
        self.description = cred_dict['description']
        self.fullname = cred_dict.get('fullName', '')
        self.displayname = cred_dict.get('displayName', '')
        self.jenkins_class = jenkins_class

    def __str__(self):
        return self.description

    def get_attributes(self):
        pass

    def get_attributes_xml(self):
        pass

    def _get_attributes_xml(self, data):
        root = ET.Element(self.jenkins_class)
        for key in data:
            value = data[key]
            if isinstance(value, dict):
                node = ET.SubElement(root, key)
                if 'stapler-class' in value:
                    node.attrib['class'] = value['stapler-class']
                for sub_key in value:
                    ET.SubElement(node, sub_key).text = value[sub_key]
            else:
                ET.SubElement(root, key).text = data[key]
        return ET.tostring(root)


class UsernamePasswordCredential(Credential):
    """
    Username and password credential

    Constructor expects following dict:
        {
            'credential_id': str,   Automatically set by jenkinsapi
            'displayName': str,     Automatically set by Jenkins
            'fullName': str,        Automatically set by Jenkins
            'typeName': str,        Automatically set by Jenkins
            'description': str,
            'userName': str,
            'password': str
        }

    When creating credential via jenkinsapi automatic fields not need to be in
    dict
    """

    def __init__(self, cred_dict):
        jenkins_class = 'com.cloudbees.plugins.credentials.impl.UsernamePasswordCredentialsImpl'
        super(UsernamePasswordCredential, self).__init__(cred_dict, jenkins_class)
        if 'typeName' in cred_dict:
            username = cred_dict['displayName'].split('/')[0]
        else:
            username = cred_dict['userName']

        self.username = username
        self.password = cred_dict.get('password', None)

    def get_attributes(self):
        """
        Used by Credentials object to create credential in Jenkins
        """
        c_id = '' if self.credential_id is None else self.credential_id
        return {
            'stapler-class': self.jenkins_class,
            'Submit': 'OK',
            'json': {
                '': '1',
                'credentials': {
                    'stapler-class': self.jenkins_class,
                    'id': c_id,
                    'username': self.username,
                    'password': self.password,
                    'description': self.description
                }
            }
        }

    def get_attributes_xml(self):
        """
        Used by Credentials object to update a credential in Jenkins
        """
        c_id = '' if self.credential_id is None else self.credential_id
        data = {
            'id': c_id,
            'username': self.username,
            'password': self.password,
            'description': self.description
        }
        return super(UsernamePasswordCredential, self)._get_attributes_xml(data)


class SecretTextCredential(Credential):
    """
    Secret text credential

    Constructor expects following dict:
        {
            'credential_id': str,   Automatically set by jenkinsapi
            'displayName': str,     Automatically set by Jenkins
            'fullName': str,        Automatically set by Jenkins
            'typeName': str,        Automatically set by Jenkins
            'description': str,
            'secret': str,
        }

    When creating credential via jenkinsapi automatic fields not need to be in
    dict
    """

    def __init__(self, cred_dict):
        jenkins_class = 'org.jenkinsci.plugins.plaincredentials.impl.StringCredentialsImpl'
        super(SecretTextCredential, self).__init__(cred_dict, jenkins_class)
        self.secret = cred_dict.get('secret', None)

    def get_attributes(self):
        """
        Used by Credentials object to create credential in Jenkins
        """
        c_id = '' if self.credential_id is None else self.credential_id
        return {
            'stapler-class': self.jenkins_class,
            'Submit': 'OK',
            'json': {
                '': '1',
                'credentials': {
                    'stapler-class': self.jenkins_class,
                    '$class': self.jenkins_class,
                    'id': c_id,
                    'secret': self.secret,
                    'description': self.description
                }
            }
        }

    def get_attributes_xml(self):
        """
        Used by Credentials object to update a credential in Jenkins
        """
        c_id = '' if self.credential_id is None else self.credential_id
        data = {
            'id': c_id,
            'secret': self.secret,
            'description': self.description
        }
        return super(SecretTextCredential, self)._get_attributes_xml(data)


class SSHKeyCredential(Credential):
    """
    SSH key credential

    Constructr expects following dict:
        {
            'credential_id': str,   Automatically set by jenkinsapi
            'displayName': str,     Automatically set by Jenkins
            'fullName': str,        Automatically set by Jenkins
            'typeName': str,        Automatically set by Jenkins
            'description': str,
            'userName': str,
            'passphrase': str,      SSH key passphrase,
            'private_key': str      Private SSH key
        }

    private_key value is parsed to find type of credential to create:

    private_key starts with -       the value is private key itself

    These credential variations are no longer supported by SSH Credentials
    plugin. jenkinsapi will raise ValueError if they are used:

    private_key starts with /       the value is a path to key
    private_key starts with ~       the value is a key from ~/.ssh

    When creating credential via jenkinsapi automatic fields not need to be in
    dict
    """

    def __init__(self, cred_dict):
        jenkins_class = 'com.cloudbees.jenkins.plugins.sshcredentials.impl.BasicSSHUserPrivateKey'
        super(SSHKeyCredential, self).__init__(cred_dict, jenkins_class)
        if 'typeName' in cred_dict:
            username = cred_dict['displayName'].split(' ')[0]
        else:
            username = cred_dict['userName']

        self.username = username
        self.passphrase = cred_dict.get('passphrase', '')

        if 'private_key' not in cred_dict or cred_dict['private_key'] is None:
            self.key_type = -1
            self.key_value = None
        elif cred_dict['private_key'].startswith('-'):
            self.key_type = 0
            self.key_value = cred_dict['private_key']
        else:
            raise ValueError('Invalid private_key value')

    @property
    def attrs(self):
        if self.key_type == 0:
            c_class = self.jenkins_class + '$DirectEntryPrivateKeySource'
        elif self.key_type == 1:
            c_class = self.jenkins_class + '$FileOnMasterPrivateKeySource'
        elif self.key_type == 2:
            c_class = self.jenkins_class + '$UsersPrivateKeySource'
        else:
            c_class = None

        attrs = {
            'value': self.key_type,
            'privateKey': self.key_value,
            'stapler-class': c_class
        }
        # We need one more attr when using the key file on master.
        if self.key_type == 1:
            attrs['privateKeyFile'] = self.key_value

        return attrs

    def get_attributes(self):
        """
        Used by Credentials object to create credential in Jenkins
        """

        c_id = '' if self.credential_id is None else self.credential_id

        return {
            'stapler-class': self.attrs['stapler-class'],
            'Submit': 'OK',
            'json': {
                '': '1',
                'credentials': {
                    'scope': 'GLOBAL',
                    'id': c_id,
                    'username': self.username,
                    'description': self.description,
                    'privateKeySource': self.attrs,
                    'passphrase': self.passphrase,
                    'stapler-class': self.jenkins_class,
                    '$class': self.jenkins_class
                }
            }
        }

    def get_attributes_xml(self):
        """
        Used by Credentials object to update a credential in Jenkins
        """
        c_id = '' if self.credential_id is None else self.credential_id
        data = {
            'id': c_id,
            'username': self.username,
            'description': self.description,
            'privateKeySource': self.attrs,
            'passphrase': self.passphrase,
        }
        return super(SSHKeyCredential, self)._get_attributes_xml(data)


class AmazonWebServicesCredentials(Credential):
    """
    AWS credential using the CloudBees AWS Credentials Plugin
    See https://wiki.jenkins.io/display/JENKINS/CloudBees+AWS+Credentials+Plugin

    Constructor expects following dict:
        {
            'credential_id': str,   Automatically set by jenkinsapi
            'displayName': str,     Automatically set by Jenkins
            'fullName': str,        Automatically set by Jenkins
            'description': str,
            'accessKey': str,
            'secretKey': str,
            'iamRoleArn': str,
            'iamMfaSerialNumber': str
        }

    When creating credential via jenkinsapi automatic fields not need to be in
    dict
    """

    def __init__(self, cred_dict):
        jenkins_class = 'com.cloudbees.jenkins.plugins.awscredentials.AWSCredentialsImpl'
        super(AmazonWebServicesCredentials, self).__init__(cred_dict, jenkins_class)

        self.access_key = cred_dict['accessKey']
        self.secret_key = cred_dict['secretKey']
        self.iam_role_arn = cred_dict.get('iamRoleArn', '')
        self.iam_mfa_serial_number = cred_dict.get('iamMfaSerialNumber', '')

    def get_attributes(self):
        """
        Used by Credentials object to create credential in Jenkins
        """
        c_id = '' if self.credential_id is None else self.credential_id
        return {
            'stapler-class': self.jenkins_class,
            'Submit': 'OK',
            'json': {
                '': '1',
                'credentials': {
                    'stapler-class': self.jenkins_class,
                    '$class': self.jenkins_class,
                    'id': c_id,
                    'accessKey': self.access_key,
                    'secretKey': self.secret_key,
                    'iamRoleArn': self.iam_role_arn,
                    'iamMfaSerialNumber': self.iam_mfa_serial_number,
                    'description': self.description
                }
            }
        }

    def get_attributes_xml(self):
        """
        Used by Credentials object to update a credential in Jenkins
        """
        c_id = '' if self.credential_id is None else self.credential_id
        data = {
            'id': c_id,
            'accessKey': self.access_key,
            'secretKey': self.secret_key,
            'iamRoleArn': self.iam_role_arn,
            'iamMfaSerialNumber': self.iam_mfa_serial_number,
            'description': self.description
        }
        return super(AmazonWebServicesCredentials, self)._get_attributes_xml(data)



import mock
# To run unittests on python 2.6 please use unittest2 library
try:
    import unittest2 as unittest
except ImportError:
    import unittest

from jenkinsapi import config
from jenkinsapi.job import Job
from jenkinsapi.jenkinsbase import JenkinsBase


# TODO: Make JOB_DATA to be one coming from Hg job
class TestHgJob(unittest.TestCase):
    JOB_DATA = {
        "actions": [],
        "description": "test job",
        "displayName": "foo",
        "displayNameOrNull": None,
        "name": "foo",
        "url": "http://halob:8080/job/foo/",
        "buildable": True,
        "builds": [
            {"number": 3, "url": "http://halob:8080/job/foo/3/"},
            {"number": 2, "url": "http://halob:8080/job/foo/2/"},
            {"number": 1, "url": "http://halob:8080/job/foo/1/"}
        ],
        "color": "blue",
        "firstBuild": {"number": 1, "url": "http://halob:8080/job/foo/1/"},
        "healthReport": [
            {"description": "Build stability: No recent builds failed.",
             "iconUrl": "health-80plus.png",
             "score": 100}
        ],
        "inQueue": False,
        "keepDependencies": False,
        # build running
        "lastBuild": {"number": 4, "url": "http://halob:8080/job/foo/4/"},
        "lastCompletedBuild": {"number": 3, "url": "http://halob:8080/job/foo/3/"},
        "lastFailedBuild": None,
        "lastStableBuild": {"number": 3, "url": "http://halob:8080/job/foo/3/"},
        "lastSuccessfulBuild": {"number": 3, "url": "http://halob:8080/job/foo/3/"},
        "lastUnstableBuild": None,
        "lastUnsuccessfulBuild": None,
        "nextBuildNumber": 4,
        "property": [],
        "queueItem": None,
        "concurrentBuild": False,
        "downstreamProjects": [],
        "scm": {},
        "upstreamProjects": []
    }

    URL_DATA = {'http://halob:8080/job/foo/%s' % config.JENKINS_API: JOB_DATA}

    def fakeGetData(self, url, *args, **kwargs):
        try:
            return TestHgJob.URL_DATA[url]
        except KeyError:
            raise Exception("Missing data for %s" % url)

    @mock.patch.object(JenkinsBase, 'get_data', fakeGetData)
    def setUp(self):
        self.J = mock.MagicMock()  # Jenkins object
        self.j = Job('http://halob:8080/job/foo/', 'foo', self.J)

    def configtree_with_branch(self):
        config_node = '''
        <project>
        <scm class="hudson.plugins.mercurial.MercurialSCM" plugin="mercurial@1.42">
        <source>http://cm5/hg/sandbox/v01.0/int</source>
        <modules/>
        <branch>testme</branch>
        <clean>false</clean>
        <browser class="hudson.plugins.mercurial.browser.HgWeb">
        <url>http://cm5/hg/sandbox/v01.0/int/</url>
        </browser>
        </scm>
        </project>
        '''
        return config_node

    def configtree_with_default_branch(self):
        config_node = '''
        <project>
        <scm class="hudson.plugins.mercurial.MercurialSCM" plugin="mercurial@1.42">
        <source>http://cm5/hg/sandbox/v01.0/int</source>
        <modules/>
        <clean>false</clean>
        <browser class="hudson.plugins.mercurial.browser.HgWeb">
        <url>http://cm5/hg/sandbox/v01.0/int/</url>
        </browser>
        </scm>
        </project>
        '''
        return config_node

    @mock.patch.object(Job, 'get_config', configtree_with_branch)
    def test_hg_attributes(self):
        expected_url = ['http://cm5/hg/sandbox/v01.0/int']
        self.assertEqual(self.j.get_scm_type(), 'hg')
        self.assertEqual(self.j.get_scm_url(), expected_url)
        self.assertEqual(self.j.get_scm_branch(), ['testme'])

    @mock.patch.object(Job, 'get_config', configtree_with_default_branch)
    def test_hg_attributes_default_branch(self):
        self.assertEqual(self.j.get_scm_branch(), ['default'])


if __name__ == '__main__':
    unittest.main()



"""
How to get build from job and query that build
"""
from __future__ import print_function
from jenkinsapi.jenkins import Jenkins


jenkins = Jenkins('http://localhost:8080')
# Print all jobs in Jenkins
print(jenkins.items())

job = jenkins.get_job("foo")
build = job.get_last_build()
print(build)

mjn = build.get_master_job_name()
print(mjn)



"""
Save this file as server.py
>>> python server.py 0.0.0.0 8001
serving on 0.0.0.0:8001

or simply

>>> python server.py
Serving on localhost:8000

You can use this to test GET and POST methods.

In order to run this example you will need the six compatibility library
install it with pip before running this script:

```
pip install six
```
"""
from six.moves import SimpleHTTPServer, socketserver
import logging
import cgi

PORT = 8081  # <-- change this to be the actual port you want to run on
INTERFACE = "localhost"


class ServerHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):

    def do_GET(self):
        logging.warning("======= GET STARTED =======")
        logging.warning(self.headers)
        SimpleHTTPServer.SimpleHTTPRequestHandler.do_GET(self)

    def do_POST(self):
        logging.warning("======= POST STARTED =======")
        logging.warning(self.headers)
        form = cgi.FieldStorage(
            fp=self.rfile,
            headers=self.headers,
            environ={'REQUEST_METHOD': 'POST',
                     'CONTENT_TYPE': self.headers['Content-Type'],
                     })
        logging.warning("======= POST VALUES =======")
        for item in form.list:
            logging.warning(item)
        logging.warning("\n")
        SimpleHTTPServer.SimpleHTTPRequestHandler.do_GET(self)


Handler = ServerHandler

httpd = socketserver.TCPServer(("", PORT), Handler)

print(
    "Serving at: http://%(interface)s:%(port)s" %
    dict(
        interface=INTERFACE or "localhost",
        port=PORT))
httpd.serve_forever()



"""
A selection of view objects used in testing.
"""

VIEW_WITH_FILTER_AND_REGEX = """
<?xml version="1.1" encoding="UTF-8"?>
<hudson.model.ListView>
  <name>%s</name>
  <filterExecutors>true</filterExecutors>
  <filterQueue>true</filterQueue>
  <properties class="hudson.model.View$PropertyList"/>
  <jobNames>
    <comparator class="hudson.util.CaseInsensitiveComparator"/>
  </jobNames>
  <jobFilters/>
  <columns>
    <hudson.views.StatusColumn/>
    <hudson.views.WeatherColumn/>
    <hudson.views.JobColumn/>
    <hudson.views.LastSuccessColumn/>
    <hudson.views.LastFailureColumn/>
    <hudson.views.LastDurationColumn/>
    <hudson.views.BuildButtonColumn/>
  </columns>
  <includeRegex>regex</includeRegex>
  <recurse>false</recurse>
</hudson.model.ListView>
""".strip()



"""
Module for jenkinsapi views
"""
import six
import logging

from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.job import Job
from jenkinsapi.custom_exceptions import NotFound


log = logging.getLogger(__name__)


class View(JenkinsBase):

    """
    View class
    """

    def __init__(self, url, name, jenkins_obj):
        self.name = name
        self.jenkins_obj = jenkins_obj
        JenkinsBase.__init__(self, url)
        self.deleted = False

    def __len__(self):
        return len(self.get_job_dict().keys())

    def __str__(self):
        return self.name

    def __repr__(self):
        return self.name

    def __getitem__(self, job_name):
        assert isinstance(job_name, str)
        api_url = self.python_api_url(self.get_job_url(job_name))
        return Job(api_url, job_name, self.jenkins_obj)

    def __contains__(self, job_name):
        """
        True if view_name is the name of a defined view
        """
        return job_name in self.keys()

    def delete(self):
        """
        Remove this view object
        """
        url = "%s/doDelete" % self.baseurl
        self.jenkins_obj.requester.post_and_confirm_status(url, data='')
        self.jenkins_obj.poll()
        self.deleted = True

    def keys(self):
        return self.get_job_dict().keys()

    def iteritems(self):
        it = six.iteritems(self.get_job_dict())

        for name, url in it:
            yield name, Job(url, name, self.jenkins_obj)

    def values(self):
        return [a[1] for a in self.iteritems()]

    def items(self):
        return [a for a in self.iteritems()]

    def _get_jobs(self):
        if 'jobs' in self._data:
            for viewdict in self._data["jobs"]:
                yield viewdict["name"], viewdict["url"]

    def get_job_dict(self):
        return dict(self._get_jobs())

    def get_job_url(self, str_job_name):
        if str_job_name in self:
            return self.get_job_dict()[str_job_name]
        else:
            # noinspection PyUnboundLocalVariable
            views_jobs = ", ".join(self.get_job_dict().keys())
            raise NotFound("Job %s is not known, available jobs"
                           " in view are: %s" % (str_job_name, views_jobs))

    def get_jenkins_obj(self):
        return self.jenkins_obj

    def add_job(self, job_name, job=None):
        """
        Add job to a view

        :param job_name: name of the job to be added
        :param job: Job object to be added
        :return: True if job has been added, False if job already exists or
         job not known to Jenkins
        """
        if not job:
            if job_name in self.get_job_dict():
                log.warning(
                    'Job %s is already in the view %s',
                    job_name, self.name)
                return False
            else:
                # Since this call can be made from nested view,
                # which doesn't have any jobs, we can miss existing job
                # Thus let's create top level Jenkins and ask him
                # http://jenkins:8080/view/CRT/view/CRT-FB/view/CRT-SCRT-1301/
                top_jenkins = self.get_jenkins_obj().get_jenkins_obj_from_url(
                    self.baseurl.split('view/')[0])
                if not top_jenkins.has_job(job_name):
                    log.error(
                        msg='Job "%s" is not known to Jenkins' %
                        job_name)
                    return False
                else:
                    job = top_jenkins.get_job(job_name)

        log.info(msg='Creating job %s in view %s' % (job_name, self.name))

        url = '%s/addJobToView' % self.baseurl
        params = {'name': job_name}

        self.get_jenkins_obj().requester.post_and_confirm_status(
            url,
            data={},
            params=params)
        self.poll()
        log.debug(msg='Job "%s" has been added to a view "%s"' %
                  (job.name, self.name))
        return True

    def remove_job(self, job_name):
        """
        Remove job from a view

        :param job_name: name of the job to be removed
        :return: True if job has been removed,
            False if job not assigned to this view
        """
        if job_name not in self:
            return False

        url = '%s/removeJobFromView' % self.baseurl
        params = {'name': job_name}

        self.get_jenkins_obj().requester.post_and_confirm_status(
            url,
            data={},
            params=params)
        self.poll()
        log.debug(
            msg='Job "%s" has been added to a view "%s"'
            % (job_name, self.name)
        )
        return True

    def _get_nested_views(self):
        for viewdict in self._data.get("views", []):
            yield viewdict["name"], viewdict["url"]

    def get_nested_view_dict(self):
        return dict(self._get_nested_views())

    def get_config_xml_url(self):
        return '%s/config.xml' % self.baseurl

    def get_config(self):
        """
        Return the config.xml from the view
        """
        url = self.get_config_xml_url()
        response = self.get_jenkins_obj().requester.get_and_confirm_status(url)
        return response.text

    def update_config(self, config):
        """
        Update the config.xml to the view
        """
        url = self.get_config_xml_url()
        config = str(config)  # cast unicode in case of Python 2

        response = self.get_jenkins_obj().requester.post_url(
            url, params={}, data=config)
        return response.text

    @property
    def views(self):
        return self.get_jenkins_obj().get_jenkins_obj_from_url(
            self.baseurl).views



import pytest
import mock
from jenkinsapi.jenkins import JenkinsBase


@pytest.fixture(scope='function')
def jenkinsbase():
    return JenkinsBase('http://localhost:8080/', poll=False)


def test_called_in__poll(jenkinsbase, monkeypatch, mocker):
    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        return {
            'description': "My jobs",
            'jobs': [{
                'name': "Foo",
                'url': "http://localhost:8080/job/Foo",
                'color': "blue",
            }],
            'name': "All",
            'property': [],
            'url': "http://localhost:8080/view/All/",
        }

    monkeypatch.setattr(JenkinsBase, '_poll', fake_poll)
    stub = mocker.stub()
    monkeypatch.setattr(JenkinsBase, 'resolve_job_folders', stub)

    jenkinsbase.poll()

    stub.assert_called_once_with(
        [
            {
                'name': "Foo",
                'url': "http://localhost:8080/job/Foo",
                'color': "blue",
            },
        ],
    )


def test_no_folders(jenkinsbase):
    jobs = [
        {
            'name': "Foo",
            'url': "http://localhost:8080/job/Foo",
            'color': "blue",
        },
        {
            'name': "Bar",
            'url': "http://localhost:8080/job/Bar",
            'color': "disabled",
        },
    ]

    assert jenkinsbase.resolve_job_folders(jobs) == [
        {
            'name': "Foo",
            'url': "http://localhost:8080/job/Foo",
            'color': "blue",
        },
        {
            'name': "Bar",
            'url': "http://localhost:8080/job/Bar",
            'color': "disabled",
        },
    ]


def test_empty_folder(jenkinsbase, monkeypatch, mocker):
    def fake_get_data(cls, url, tree=None):  # pylint: disable=unused-argument
        return {'jobs': []}

    monkeypatch.setattr(JenkinsBase, 'get_data', fake_get_data)
    spy = mocker.spy(jenkinsbase, 'get_data')

    jobs = [
        {
            'name': "Folder1",
            'url': "http://localhost:8080/job/Folder1",
        },
    ]

    assert jenkinsbase.resolve_job_folders(jobs) == []
    spy.assert_called_once_with(
        'http://localhost:8080/job/Folder1/api/python',
        tree='jobs[name,color]'
    )


def test_folder_job_mix(jenkinsbase, monkeypatch, mocker):
    def fake_get_data(cls, url, tree=None):  # pylint: disable=unused-argument
        return {
            'jobs': [
                {
                    'name': "Bar",
                    'url': "http://localhost:8080/job/Folder1/job/Bar",
                    'color': "disabled",
                }
            ]
        }

    monkeypatch.setattr(JenkinsBase, 'get_data', fake_get_data)
    spy = mocker.spy(jenkinsbase, 'get_data')

    jobs = [
        {
            'name': "Foo",
            'url': "http://localhost:8080/job/Foo",
            'color': "blue",
        },
        {
            'name': "Folder1",
            'url': "http://localhost:8080/job/Folder1",
        },
    ]

    assert jenkinsbase.resolve_job_folders(jobs) == [
        {
            'name': "Foo",
            'url': "http://localhost:8080/job/Foo",
            'color': "blue",
        },
        {
            'name': "Bar",
            'url': "http://localhost:8080/job/Folder1/job/Bar",
            'color': "disabled",
        }
    ]

    spy.assert_called_once_with(
        'http://localhost:8080/job/Folder1/api/python',
        tree='jobs[name,color]'
    )


def test_multiple_folders(jenkinsbase, monkeypatch, mocker):
    def fake_get_data(cls, url, tree=None):  # pylint: disable=unused-argument
        # first call
        if 'Folder1' in url:
            return {'jobs': [
                {
                    'name': "Foo",
                    'url': "http://localhost:8080/job/Folder1/job/Foo",
                    'color': "disabled",
                },
            ]}

        if 'Folder2' in url:
            # second call
            return {'jobs': [
                {
                    'name': "Bar",
                    'url': "http://localhost:8080/job/Folder2/job/Bar",
                    'color': "blue",
                },
            ]}

    monkeypatch.setattr(JenkinsBase, 'get_data', fake_get_data)
    spy = mocker.spy(jenkinsbase, 'get_data')

    jobs = [
        {
            'name': "Folder1",
            'url': "http://localhost:8080/job/Folder1",
        },
        {
            'name': "Folder2",
            'url': "http://localhost:8080/job/Folder2",
        },
    ]

    assert jenkinsbase.resolve_job_folders(jobs) == [
        {
            'name': "Foo",
            'url': "http://localhost:8080/job/Folder1/job/Foo",
            'color': "disabled",
        },
        {
            'name': "Bar",
            'url': "http://localhost:8080/job/Folder2/job/Bar",
            'color': "blue",
        },
    ]

    assert spy.call_args_list == [
        mock.call(
            'http://localhost:8080/job/Folder1/api/python',
            tree='jobs[name,color]'
        ),
        mock.call(
            'http://localhost:8080/job/Folder2/api/python',
            tree='jobs[name,color]'
        ),
    ]


def test_multiple_folder_levels(jenkinsbase, monkeypatch, mocker):
    def fake_get_data(cls, url, tree=None):  # pylint: disable=unused-argument
        if 'Folder1' in url and 'Folder2' not in url:
            # first call
            return {'jobs': [
                {
                    'name': "Bar",
                    'url': "http://localhost:8080/job/Folder1/job/Bar",
                    'color': "disabled",
                },
                {
                    'name': "Folder2",
                    'url': "http://localhost:8080/job/Folder1/job/Folder2",
                }
            ]}

        if 'Folder2' in url:
            # second call
            return {
                'jobs': [
                    {
                        'name': "Baz",
                        'url': (
                            "http://localhost:8080/job/Folder1/"
                            "job/Folder2/job/Baz"
                        ),
                        'color': "disabled",
                    },
                ]
            }

    monkeypatch.setattr(JenkinsBase, 'get_data', fake_get_data)
    spy = mocker.spy(jenkinsbase, 'get_data')

    jobs = [
        {
            'name': "Foo",
            'url': "http://localhost:8080/job/Foo",
            'color': "blue",
        },
        {
            'name': "Folder1",
            'url': "http://localhost:8080/job/Folder1",
        },
    ]

    assert jenkinsbase.resolve_job_folders(jobs) == [
        {
            'name': "Foo",
            'url': "http://localhost:8080/job/Foo",
            'color': "blue",
        },
        {
            'name': "Bar",
            'url': "http://localhost:8080/job/Folder1/job/Bar",
            'color': "disabled",
        },
        {
            'name': "Baz",
            'url': ("http://localhost:8080/job/Folder1"
                    "/job/Folder2/job/Baz"),
            'color': "disabled",
        },
    ]

    assert spy.call_args_list == [
        mock.call(
            'http://localhost:8080/job/Folder1/api/python',
            tree='jobs[name,color]'
        ),
        mock.call(
            'http://localhost:8080/job/Folder1'
            '/job/Folder2/api/python',
            tree='jobs[name,color]'
        ),
    ]



import pytest
import mock
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.executors import Executors
from jenkinsapi.executor import Executor


DATAM = {
    'assignedLabels': [{}],
    'description': None,
    'jobs': [],
    'mode': 'NORMAL',
    'nodeDescription': 'the master Jenkins node',
    'nodeName': '',
    'numExecutors': 2,
    'overallLoad': {},
    'primaryView': {'name': 'All', 'url': 'http://localhost:8080/'},
    'quietingDown': False,
    'slaveAgentPort': 0,
    'unlabeledLoad': {},
    'useCrumbs': False,
    'useSecurity': False,
    'views': [
        {'name': 'All', 'url': 'http://localhost:8080/'},
        {'name': 'BigMoney', 'url': 'http://localhost:8080/view/BigMoney/'}
    ]
}

DATA0 = {
    "actions": [
    ],
    "displayName": "host0.host.com",
    "executors": [
        {},
        {},
        {},
        {},
        {},
        {},
        {},
        {}
    ],
    "icon": "computer.png",
    "idle": False,
    "jnlpAgent": True,
    "launchSupported": False,
    "loadStatistics": {

    },
    "manualLaunchAllowed": True,
    "monitorData": {
        "hudson.node_monitors.SwapSpaceMonitor": {
            "availablePhysicalMemory": 8462417920,
            "availableSwapSpace": 0,
            "totalPhysicalMemory": 75858042880,
            "totalSwapSpace": 0
        },
        "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
        "hudson.node_monitors.ResponseTimeMonitor": {
            "average": 2
        },
        "hudson.node_monitors.TemporarySpaceMonitor": {
            "path": "/tmp",
            "size": 430744551424
        },
        "hudson.node_monitors.DiskSpaceMonitor": {
            "path": "/data/jenkins",
            "size": 1214028627968
        },
        "hudson.node_monitors.ClockMonitor": {
            "diff": 1
        }
    },
    "numExecutors": 8,
    "offline": False,
    "offlineCause": None,
    "offlineCauseReason": "",
    "oneOffExecutors": [
        {},
        {}
    ],
    "temporarilyOffline": False
}

DATA1 = {
    "actions": [
    ],
    "displayName": "host1.host.com",
    "executors": [
        {},
        {}
    ],
    "icon": "computer.png",
    "idle": False,
    "jnlpAgent": True,
    "launchSupported": False,
    "loadStatistics": {
    },
    "manualLaunchAllowed": True,
    "monitorData": {
        "hudson.node_monitors.SwapSpaceMonitor": {
            "availablePhysicalMemory": 8462417920,
            "availableSwapSpace": 0,
            "totalPhysicalMemory": 75858042880,
            "totalSwapSpace": 0
        },
        "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
        "hudson.node_monitors.ResponseTimeMonitor": {
            "average": 2
        },
        "hudson.node_monitors.TemporarySpaceMonitor": {
            "path": "/tmp",
            "size": 430744551424
        },
        "hudson.node_monitors.DiskSpaceMonitor": {
            "path": "/data/jenkins",
            "size": 1214028627968
        },
        "hudson.node_monitors.ClockMonitor": {
            "diff": 1
        }
    },
    "numExecutors": 2,
    "offline": False,
    "offlineCause": None,
    "offlineCauseReason": "",
    "oneOffExecutors": [
        {},
        {}
    ],
    "temporarilyOffline": False
}

DATA2 = {
    "actions": [
    ],
    "displayName": "host2.host.com",
    "executors": [
        {},
        {},
        {},
        {}
    ],
    "icon": "computer.png",
    "idle": False,
    "jnlpAgent": True,
    "launchSupported": False,
    "loadStatistics": {
    },
    "manualLaunchAllowed": True,
    "monitorData": {
        "hudson.node_monitors.SwapSpaceMonitor": {
            "availablePhysicalMemory": 8462417920,
            "availableSwapSpace": 0,
            "totalPhysicalMemory": 75858042880,
            "totalSwapSpace": 0
        },
        "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
        "hudson.node_monitors.ResponseTimeMonitor": {
            "average": 2
        },
        "hudson.node_monitors.TemporarySpaceMonitor": {
            "path": "/tmp",
            "size": 430744551424
        },
        "hudson.node_monitors.DiskSpaceMonitor": {
            "path": "/data/jenkins",
            "size": 1214028627968
        },
        "hudson.node_monitors.ClockMonitor": {
            "diff": 1
        }
    },
    "numExecutors": 4,
    "offline": False,
    "offlineCause": None,
    "offlineCauseReason": "",
    "oneOffExecutors": [
        {},
        {}
    ],
    "temporarilyOffline": False
}

DATA3 = {
    "actions": [
    ],
    "displayName": "host3.host.com",
    "executors": [
        {}
    ],
    "icon": "computer.png",
    "idle": False,
    "jnlpAgent": True,
    "launchSupported": False,
    "loadStatistics": {},
    "manualLaunchAllowed": True,
    "monitorData": {
        "hudson.node_monitors.SwapSpaceMonitor": {
            "availablePhysicalMemory": 8462417920,
            "availableSwapSpace": 0,
            "totalPhysicalMemory": 75858042880,
            "totalSwapSpace": 0
        },
        "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
        "hudson.node_monitors.ResponseTimeMonitor": {
            "average": 2
        },
        "hudson.node_monitors.TemporarySpaceMonitor": {
            "path": "/tmp",
            "size": 430744551424
        },
        "hudson.node_monitors.DiskSpaceMonitor": {
            "path": "/data/jenkins",
            "size": 1214028627968
        },
        "hudson.node_monitors.ClockMonitor": {
            "diff": 1
        }
    },
    "numExecutors": 1,
    "offline": False,
    "offlineCause": None,
    "offlineCauseReason": "",
    "oneOffExecutors": [
        {},
        {}
    ],
    "temporarilyOffline": False
}

EXEC0 = {
    "currentExecutable": {
        "number": 4168,
        "url": "http://localhost:8080/job/testjob/4168/"
    },
    "currentWorkUnit": {},
    "idle": False,
    "likelyStuck": False,
    "number": 0,
    "progress": 48
}

EXEC1 = {
    "currentExecutable": None,
    "currentWorkUnit": None,
    "idle": True,
    "likelyStuck": False,
    "number": 0,
    "progress": -1
}


@pytest.fixture(scope='function')
def jenkins(monkeypatch):
    def fake_poll(cls, tree=None):      # pylint: disable=unused-argument
        return DATAM

    monkeypatch.setattr(Jenkins, '_poll', fake_poll)
    return Jenkins('http://localhost:8080')


def test_repr(jenkins):
    # Can we produce a repr string for this object
    assert repr(jenkins)


def test_check_url(jenkins):
    assert jenkins.baseurl == 'http://localhost:8080'


def test_get_executors(jenkins, monkeypatch):
    def fake_poll_extr(cls, tree=None):  # pylint: disable=unused-argument
        return EXEC0

    def fake_poll_extrs(cls, tree=None):  # pylint: disable=unused-argument
        return DATA3

    monkeypatch.setattr(Executor, '_poll', fake_poll_extr)
    monkeypatch.setattr(Executors, '_poll', fake_poll_extrs)

    exec_info = jenkins.get_executors(DATA3['displayName'])

    assert isinstance(exec_info, object)
    assert isinstance(repr(exec_info), str)

    for ex in exec_info:
        assert ex.get_progress() == 48, 'Should return 48 %'


def testis_idle(jenkins, monkeypatch):
    def fake_poll_extr(cls, tree=None):  # pylint: disable=unused-argument
        return EXEC1

    def fake_poll_extrs(cls, tree=None):  # pylint: disable=unused-argument
        return DATA3

    monkeypatch.setattr(Executor, '_poll', fake_poll_extr)
    monkeypatch.setattr(Executors, '_poll', fake_poll_extrs)

    exec_info = jenkins.get_executors('host3.host.com')

    assert isinstance(exec_info, object)
    for ex in exec_info:
        assert ex.get_progress() == -1, 'Should return 48 %'
        assert ex.is_idle() is True, 'Should return True'
        assert repr(ex) == '<jenkinsapi.executor.Executor host3.host.com 0>'


@mock.patch.object(Executor, '_poll')
def test_likely_stuck(jenkins, monkeypatch):
    def fake_poll_extr(cls, tree=None):  # pylint: disable=unused-argument
        return EXEC0

    monkeypatch.setattr(Executor, '_poll', fake_poll_extr)

    baseurl = 'http://localhost:8080/computer/host0.host.com/executors/0'
    nodename = 'host0.host.com'
    single_executer = Executor(baseurl, nodename, jenkins, '0')
    assert single_executer.likely_stuck() is False


def test_get_current_executable(jenkins, monkeypatch):
    def fake_poll_extr(cls, tree=None):  # pylint: disable=unused-argument
        return EXEC0

    monkeypatch.setattr(Executor, '_poll', fake_poll_extr)

    baseurl = 'http://localhost:8080/computer/host0.host.com/executors/0'
    nodename = 'host0.host.com'
    single_executer = Executor(baseurl, nodename, jenkins, '0')
    assert single_executer.get_current_executable()['number'] == 4168
    assert single_executer.get_current_executable()['url'] == \
        'http://localhost:8080/job/testjob/4168/'



# -*- coding: utf-8 -*-
import six
from jenkinsapi_utils.compat import (
    to_string,
    needs_encoding,
)


def test_needs_encoding_py2():
    if six.PY2:
        unicode_str = u'юникод'
        assert needs_encoding(unicode_str)

    assert not needs_encoding('string')
    assert not needs_encoding(5)
    assert not needs_encoding(['list', 'of', 'strings'])


def test_to_string():
    assert isinstance(to_string(5), str)
    assert isinstance(to_string('string'), str)
    assert isinstance(to_string(['list', 'of', 'strings']), str)
    assert isinstance(to_string(u'unicode'), str)



"""
How to create slaves/nodes
"""
import logging
import requests
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.utils.requester import Requester
requests.packages.urllib3.disable_warnings()

log_level = getattr(logging, 'DEBUG')
logging.basicConfig(level=log_level)
logger = logging.getLogger()

jenkins_url = "http://localhost:8080/"
username = "default_user"  # In case Jenkins requires authentication
password = "default_password"

jenkins = Jenkins(jenkins_url,
                  requester=Requester(username, password,
                                      baseurl=jenkins_url, ssl_verify=False))

# Create JNLP(Java Webstart) slave
node_dict = {
    'num_executors': 1,                     # Number of executors
    'node_description': 'Test JNLP Node',   # Just a user friendly text
    'remote_fs': '/tmp',                    # Remote workspace location
    'labels': 'my_new_node',                # Space separated labels string
    'exclusive': True                       # Only run jobs assigned to it
}
new_jnlp_node = jenkins.nodes.create_node('My new webstart node', node_dict)

node_dict = {
    'num_executors': 1,
    'node_description': 'Test SSH Node',
    'remote_fs': '/tmp',
    'labels': 'new_node',
    'exclusive': True,
    'host': 'localhost',                         # Remote hostname
    'port': 22,                                  # Remote post, usually 22
    'credential_description': 'localhost cred',  # Credential to use
                                                 # [Mandatory for SSH node!]
                                                 # (see Credentials example)
    'jvm_options': '-Xmx2000M',                  # JVM parameters
    'java_path': '/bin/java',                    # Path to java
    'prefix_start_slave_cmd': '',
    'suffix_start_slave_cmd': '',
    'max_num_retries': 0,
    'retry_wait_time': 0,
    'retention': 'OnDemand',                     # Change to 'Always' for
                                                 # immediate slave launch
    'ondemand_delay': 1,
    'ondemand_idle_delay': 5,
    'env': [                                     # Environment variables
        {
            'key': 'TEST',
            'value': 'VALUE'
        },
        {
            'key': 'TEST2',
            'value': 'value2'
        }
    ]
}
new_ssh_node = jenkins.nodes.create_node('My new SSH node', node_dict)

# Take this slave offline
if new_ssh_node.is_online():
    new_ssh_node.toggle_temporarily_offline()

    # Take this slave back online
    new_ssh_node.toggle_temporarily_offline()

# Get a list of all slave names
slave_names = jenkins.nodes.keys()

# Get Node object
my_node = jenkins.nodes['My new SSH node']
# Take this slave offline
my_node.set_offline()

# Delete slaves
del jenkins.nodes['My new webstart node']
del jenkins.nodes['My new SSH node']



"""
This example shows how to add new command to "Shell" build step
"""
from __future__ import print_function
import xml.etree.ElementTree as et
from jenkinsapi.jenkins import Jenkins

J = Jenkins('http://localhost:8080')
EMPTY_JOB_CONFIG = '''
<?xml version='1.0' encoding='UTF-8'?>
<project>
  <actions>jkkjjk</actions>
  <description></description>
  <keepDependencies>false</keepDependencies>
  <properties/>
  <scm class="hudson.scm.NullSCM"/>
  <canRoam>true</canRoam>
  <disabled>false</disabled>
  <blockBuildWhenDownstreamBuilding>false</blockBuildWhenDownstreamBuilding>
  <blockBuildWhenUpstreamBuilding>false</blockBuildWhenUpstreamBuilding>
  <triggers class="vector"/>
  <concurrentBuild>false</concurrentBuild>
  <builders/>
  <publishers/>
  <buildWrappers/>
</project>
'''

jobname = 'foo_job'
new_job = J.create_job(jobname, EMPTY_JOB_CONFIG)
new_conf = new_job.get_config()

root = et.fromstring(new_conf.strip())

builders = root.find('builders')
shell = et.SubElement(builders, 'hudson.tasks.Shell')
command = et.SubElement(shell, 'command')
command.text = "ls"

print(et.tostring(root))
J[jobname].update_config(et.tostring(root))



import pytest
import hashlib
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.fingerprint import Fingerprint
from jenkinsapi.utils.requester import Requester
from requests.exceptions import HTTPError


@pytest.fixture(scope='function')
def jenkins(monkeypatch):
    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(Jenkins, '_poll', fake_poll)

    return Jenkins('http://localhost:8080',
                   username='foouser', password='foopassword')


@pytest.fixture(scope='module')
def dummy_md5():
    md = hashlib.md5()
    md.update("some dummy string".encode('ascii'))
    return md.hexdigest()


def test_object_creation(jenkins, dummy_md5, monkeypatch):
    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        return {}

    monkeypatch.setattr(JenkinsBase, '_poll', fake_poll)
    fp_instance = Fingerprint('http://foo:8080', dummy_md5, jenkins)

    assert isinstance(fp_instance, Fingerprint)
    assert str(fp_instance) == dummy_md5
    assert fp_instance.valid()


def test_valid_for_404(jenkins, dummy_md5, monkeypatch):
    class FakeResponse(object):
        status_code = 404
        text = '{}'

    class FakeHTTPError(HTTPError):
        def __init__(self):
            self.response = FakeResponse()

    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        raise FakeHTTPError()

    monkeypatch.setattr(JenkinsBase, '_poll', fake_poll)

    def fake_get_url(
            url,  # pylint: disable=unused-argument
            params=None,  # pylint: disable=unused-argument
            headers=None,  # pylint: disable=unused-argument
            allow_redirects=True,  # pylint: disable=unused-argument
            stream=False):  # pylint: disable=unused-argument

        return FakeResponse()

    monkeypatch.setattr(Requester, 'get_url', fake_get_url)

    fingerprint = Fingerprint('http://foo:8080', dummy_md5, jenkins)
    assert fingerprint.valid() is True


def test_invalid_for_401(jenkins, dummy_md5, monkeypatch):
    class FakeResponse(object):
        status_code = 401
        text = '{}'

    class FakeHTTPError(HTTPError):
        def __init__(self):
            self.response = FakeResponse()

    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        raise FakeHTTPError()

    monkeypatch.setattr(JenkinsBase, '_poll', fake_poll)

    def fake_get_url(
            url,  # pylint: disable=unused-argument
            params=None,  # pylint: disable=unused-argument
            headers=None,  # pylint: disable=unused-argument
            allow_redirects=True,  # pylint: disable=unused-argument
            stream=False):  # pylint: disable=unused-argument

        return FakeResponse()

    monkeypatch.setattr(Requester, 'get_url', fake_get_url)

    fingerprint = Fingerprint('http://foo:8080', dummy_md5, jenkins)
    assert fingerprint.valid() is not True



import six
import pytest
from . import configs
from jenkinsapi.build import Build
from jenkinsapi.job import Job


@pytest.fixture(scope='function')
def jenkins(mocker):
    return mocker.MagicMock()


@pytest.fixture(scope='function')
def job(monkeypatch, jenkins):
    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        return configs.JOB_DATA

    monkeypatch.setattr(Job, '_poll', fake_poll)

    fake_job = Job('http://', 'Fake_Job', jenkins)
    return fake_job


@pytest.fixture(scope='function')
def build(job, monkeypatch):
    def fake_poll(cls, tree=None):   # pylint: disable=unused-argument
        return configs.BUILD_SCM_DATA

    monkeypatch.setattr(Build, '_poll', fake_poll)

    return Build('http://', 97, job)


def test_git_scm(build):
    """
    Can we extract git build revision data from a build object?
    """
    assert isinstance(build.get_revision(), six.string_types)
    assert build.get_revision() == '7def9ed6e92580f37d00e4980c36c4d36e68f702'


def test_git_revision_branch(build):
    """
    Can we extract git build branch from a build object?
    """
    assert isinstance(build.get_revision_branch(), list)
    assert len(build.get_revision_branch()) == 1
    assert isinstance(build.get_revision_branch()[0], dict)
    assert build.get_revision_branch()[0]['SHA1'] == \
        '7def9ed6e92580f37d00e4980c36c4d36e68f702'
    assert build.get_revision_branch()[0]['name'] == 'origin/unstable'


def test_git_repo_url(build):
    """
     Can we Extract git repo url for a given build
    """
    assert isinstance(build.get_repo_url(), str)
    assert build.get_repo_url() == 'https://github.com/salimfadhley/jenkinsapi.git'



from __future__ import print_function
import pytest
import requests
from jenkinsapi.jenkins import Requester
from jenkinsapi.custom_exceptions import JenkinsAPIException
from mock import patch


def test_no_parameters_uses_default_values():
    req = Requester()
    assert isinstance(req, Requester)
    assert req.username is None
    assert req.password is None
    assert req.ssl_verify
    assert req.cert is None
    assert req.base_scheme is None
    assert req.timeout == 10


def test_all_named_parameters():
    req = Requester(username='foo', password='bar', ssl_verify=False,
                    cert='foobar', baseurl='http://dummy', timeout=5)
    assert isinstance(req, Requester)
    assert req.username == 'foo'
    assert req.password == 'bar'
    assert not req.ssl_verify
    assert req.cert == 'foobar'
    assert req.base_scheme == 'http', 'dummy'
    assert req.timeout == 5


def test_mix_one_unnamed_named_parameters():
    req = Requester('foo', password='bar', ssl_verify=False, cert='foobar',
                    baseurl='http://dummy', timeout=5)
    assert isinstance(req, Requester)
    assert req.username == 'foo'
    assert req.password == 'bar'
    assert not req.ssl_verify
    assert req.cert == 'foobar'
    assert req.base_scheme == 'http', 'dummy'
    assert req.timeout == 5


def test_mix_two_unnamed_named_parameters():
    req = Requester('foo', 'bar', ssl_verify=False, cert='foobar',
                    baseurl='http://dummy', timeout=5)
    assert isinstance(req, Requester)
    assert req.username == 'foo'
    assert req.password == 'bar'
    assert not req.ssl_verify
    assert req.cert == 'foobar'
    assert req.base_scheme == 'http', 'dummy'
    assert req.timeout == 5


def test_mix_three_unnamed_named_parameters():
    req = Requester('foo', 'bar', False, cert='foobar', baseurl='http://dummy',
                    timeout=5)
    assert isinstance(req, Requester)
    assert req.username == 'foo'
    assert req.password == 'bar'
    assert not req.ssl_verify
    assert req.cert == 'foobar'
    assert req.base_scheme == 'http', 'dummy'
    assert req.timeout == 5


def test_mix_four_unnamed_named_parameters():
    req = Requester('foo', 'bar', False, 'foobar', baseurl='http://dummy',
                    timeout=5)
    assert isinstance(req, Requester)
    assert req.username == 'foo'
    assert req.password == 'bar'
    assert not req.ssl_verify
    assert req.cert == 'foobar'
    assert req.base_scheme == 'http', 'dummy'
    assert req.timeout == 5


def test_mix_five_unnamed_named_parameters():
    req = Requester('foo', 'bar', False, 'foobar', 'http://dummy', timeout=5)
    assert isinstance(req, Requester)
    assert req.username == 'foo'
    assert req.password == 'bar'
    assert not req.ssl_verify
    assert req.cert == 'foobar'
    assert req.base_scheme == 'http', 'dummy'
    assert req.timeout == 5


def test_all_unnamed_parameters():
    req = Requester('foo', 'bar', False, 'foobar', 'http://dummy', 5)
    assert isinstance(req, Requester)
    assert req.username == 'foo'
    assert req.password == 'bar'
    assert not req.ssl_verify
    assert req.cert == 'foobar'
    assert req.base_scheme == 'http', 'dummy'
    assert req.timeout == 5


def test_to_much_unnamed_parameters_raises_error():
    with pytest.raises(Exception):
        Requester('foo', 'bar', False, 'foobar', 'http://dummy', 5, 'test')


def test_username_without_password_raises_error():
    with pytest.raises(Exception):
        Requester(username='foo')
        Requester('foo')


def test_password_without_username_raises_error():
    with pytest.raises(AssertionError):
        Requester(password='bar')


def test_get_request_dict_auth():
    req = Requester('foo', 'bar')

    req_return = req.get_request_dict(
        params={},
        data=None,
        headers=None
    )
    assert isinstance(req_return, dict)
    assert req_return.get('auth')
    assert req_return['auth'] == ('foo', 'bar')


@patch('jenkinsapi.jenkins.Requester.AUTH_COOKIE', 'FAKE')
def test_get_request_dict_cookie():
    req = Requester('foo', 'bar')

    req_return = req.get_request_dict(
        params={},
        data=None,
        headers=None
    )
    assert isinstance(req_return, dict)
    assert req_return.get('headers')
    assert req_return.get('headers').get('Cookie')
    assert req_return.get('headers').get('Cookie') == 'FAKE'


@patch('jenkinsapi.jenkins.Requester.AUTH_COOKIE', 'FAKE')
def test_get_request_dict_updatecookie():
    req = Requester('foo', 'bar')

    req_return = req.get_request_dict(
        params={},
        data=None,
        headers={'key': 'value'}
    )
    assert isinstance(req_return, dict)
    assert req_return.get('headers')
    assert req_return.get('headers').get('key')
    assert req_return.get('headers').get('key') == 'value'
    assert req_return.get('headers').get('Cookie')
    assert req_return.get('headers').get('Cookie') == 'FAKE'


def test_get_request_dict_nocookie():
    req = Requester('foo', 'bar')

    req_return = req.get_request_dict(
        params={},
        data=None,
        headers=None
    )
    assert isinstance(req_return, dict)
    assert not req_return.get('headers')


def test_get_request_dict_wrong_params():
    req = Requester('foo', 'bar')

    with pytest.raises(AssertionError) as na:
        req.get_request_dict(
            params='wrong',
            data=None,
            headers=None
        )
    assert "Params must be a dict, got 'wrong'" in str(na.value)


def test_get_request_dict_correct_params():
    req = Requester('foo', 'bar')

    req_return = req.get_request_dict(
        params={'param': 'value'},
        data=None,
        headers=None
    )

    assert isinstance(req_return, dict)
    assert req_return.get('params')
    assert req_return['params'] == {'param': 'value'}


def test_get_request_dict_wrong_headers():
    req = Requester('foo', 'bar')

    with pytest.raises(AssertionError) as na:
        req.get_request_dict(
            params={},
            data=None,
            headers='wrong'
        )
    assert "headers must be a dict, got 'wrong'" in str(na.value)


def test_get_request_dict_correct_headers():
    req = Requester('foo', 'bar')

    req_return = req.get_request_dict(
        params={'param': 'value'},
        data=None,
        headers={'header': 'value'}
    )

    assert isinstance(req_return, dict)
    assert req_return.get('headers')
    assert req_return['headers'] == {'header': 'value'}


def test_get_request_dict_data_passed():
    req = Requester('foo', 'bar')

    req_return = req.get_request_dict(
        params={'param': 'value'},
        data='some data',
        headers={'header': 'value'}
    )

    assert isinstance(req_return, dict)
    assert req_return.get('data')
    assert req_return['data'] == 'some data'


def test_get_request_dict_data_not_passed():
    req = Requester('foo', 'bar')

    req_return = req.get_request_dict(
        params={'param': 'value'},
        data=None,
        headers={'header': 'value'}
    )

    assert isinstance(req_return, dict)
    assert req_return.get('data') is None


def test_get_url_get(monkeypatch):
    def fake_get(*args, **kwargs):  # pylint: disable=unused-argument
        return 'SUCCESS'

    monkeypatch.setattr(requests.Session, 'get', fake_get)

    req = Requester('foo', 'bar')
    response = req.get_url(
        'http://dummy',
        params={'param': 'value'},
        headers=None)

    assert response == 'SUCCESS'


def test_get_url_post(monkeypatch):
    def fake_post(*args, **kwargs):  # pylint: disable=unused-argument
        return 'SUCCESS'

    monkeypatch.setattr(requests.Session, 'post', fake_post)

    req = Requester('foo', 'bar')
    response = req.post_url(
        'http://dummy',
        params={'param': 'value'},
        headers=None)

    assert response == 'SUCCESS'


def test_post_xml_empty_xml(monkeypatch):
    def fake_post(*args, **kwargs):  # pylint: disable=unused-argument
        return 'SUCCESS'

    monkeypatch.setattr(requests.Session, 'post', fake_post)

    req = Requester('foo', 'bar')
    with pytest.raises(AssertionError):
        req.post_xml_and_confirm_status(
            url='http://dummy',
            params={'param': 'value'},
            data=None
        )


def test_post_xml_and_confirm_status_some_xml(monkeypatch):
    class FakeResponse(requests.Response):
        def __init__(self, *args, **kwargs):  # pylint: disable=unused-argument
            self.status_code = 200

    def fake_post(*args, **kwargs):  # pylint: disable=unused-argument
        return FakeResponse()

    monkeypatch.setattr(requests.Session, 'post', fake_post)

    req = Requester('foo', 'bar')
    ret = req.post_xml_and_confirm_status(
        url='http://dummy',
        params={'param': 'value'},
        data='<xml/>'
    )
    assert isinstance(ret, requests.Response)


def test_post_and_confirm_status_empty_data(monkeypatch):
    def fake_post(*args, **kwargs):  # pylint: disable=unused-argument
        return 'SUCCESS'

    monkeypatch.setattr(requests.Session, 'post', fake_post)

    req = Requester('foo', 'bar')
    with pytest.raises(AssertionError):
        req.post_and_confirm_status(
            url='http://dummy',
            params={'param': 'value'},
            data=None
        )


def test_post_and_confirm_status_some_data(monkeypatch):
    class FakeResponse(requests.Response):
        def __init__(self, *args, **kwargs):  # pylint: disable=unused-argument
            self.status_code = 200

    def fake_post(*args, **kwargs):  # pylint: disable=unused-argument
        return FakeResponse()

    monkeypatch.setattr(requests.Session, 'post', fake_post)

    req = Requester('foo', 'bar')
    ret = req.post_and_confirm_status(
        url='http://dummy',
        params={'param': 'value'},
        data='some data'
    )
    assert isinstance(ret, requests.Response)


def test_post_and_confirm_status_bad_result(monkeypatch):
    class FakeResponse(object):
        def __init__(self, *args, **kwargs):  # pylint: disable=unused-argument
            self.status_code = 500
            self.url = 'http://dummy'
            self.text = 'something'

    def fake_post(*args, **kwargs):  # pylint: disable=unused-argument
        return FakeResponse()

    monkeypatch.setattr(requests.Session, 'post', fake_post)

    req = Requester('foo', 'bar')
    with pytest.raises(JenkinsAPIException):
        req.post_and_confirm_status(
            url='http://dummy',
            params={'param': 'value'},
            data='some data'
        )


def test_get_and_confirm_status(monkeypatch):
    class FakeResponse(requests.Response):
        def __init__(self, *args, **kwargs):  # pylint: disable=unused-argument
            self.status_code = 200

    def fake_get(*args, **kwargs):  # pylint: disable=unused-argument
        return FakeResponse()

    monkeypatch.setattr(requests.Session, 'get', fake_get)

    req = Requester('foo', 'bar')
    ret = req.get_and_confirm_status(
        url='http://dummy',
        params={'param': 'value'}
    )
    assert isinstance(ret, requests.Response)


def test_get_and_confirm_status_bad_result(monkeypatch):
    class FakeResponse(object):
        def __init__(self, *args, **kwargs):  # pylint: disable=unused-argument
            self.status_code = 500
            self.url = 'http://dummy'
            self.text = 'something'

    def fake_get(*args, **kwargs):  # pylint: disable=unused-argument
        return FakeResponse()

    monkeypatch.setattr(requests.Session, 'get', fake_get)

    req = Requester('foo', 'bar', baseurl='http://dummy')
    with pytest.raises(JenkinsAPIException):
        req.get_and_confirm_status(
            url='http://dummy',
            params={'param': 'value'}
        )
